from .katcr import Katcr  # noqa
from .tpb import ThePirateBay  # noqa
from .nyaasi import NyaaSi  # noqa
from .skytorrents import Skytorrents  # noqa
from .eztv import Eztv  # noqa
from .jackett import Jackett  # noqa
